from typing import List, Union

class AvatarFrameNotFound:
	"""`AvatarFrameNotFound` - Class representing a avatar frame not found."""
	def __init__(self):
		pass

class AvatarFrame:
	"""
	`AvatarFrame` - Class representing a avatar frame.

	`Attributes`:
	- `_data` - The data containing the avatar frame.
	
	`Properties`:
	- `status` - The status of the request.
	- `ownership_status` - The ownership status of the avatar frame.
	- `version` - The version of the avatar frame.
	- `resource_url` - The resource url of the avatar frame.
	- `name` - The name of the avatar frame.
	- `icon` - The icon of the avatar frame.
	- `frame_type` - The frame type of the avatar frame.
	- `frame_id` - The frame id of the avatar frame.
	"""
	def __init__(self, data: dict):
		self._data = data

	@property
	def status(self) -> int:
		"""
		`status` - The status of the request.
		
		`Returns`:
		- `int` - The status of the request.
		"""
		return self._data.get('status')

	@property
	def ownership_status(self) -> str:
		"""
		`ownership_status` - The ownership status of the avatar frame.

		`Returns`:
		- `str` - The ownership status of the avatar frame.
		"""
		return self._data.get('ownershipStatus')

	@property
	def version(self) -> int:
		"""
		`version` - The version of the avatar frame.

		`Returns`:
		- `int` - The version of the avatar frame.
		"""
		return self._data.get('version')
		
	@property
	def resource_url(self) -> str:
		"""
		`resource_url` - The resource url of the avatar frame.

		`Returns`:
		- `str` - The resource url of the avatar frame.
		"""
		return self._data.get('resourceUrl')

	@property
	def name(self) -> str:
		"""
		`name` - The name of the avatar frame.

		`Returns`:
		- `str` - The name of the avatar frame.
		"""
		return self._data.get('name')

	@property
	def icon(self) -> str:
		"""
		`icon` - The icon of the avatar frame.
		
		`Returns`:
		- `str` - The icon of the avatar frame.
		"""
		return self._data.get('icon')

	@property
	def frame_type(self) -> int:
		"""
		`frame_type` - The frame type of the avatar frame.
		
		`Returns`:
		- `int` - The frame type of the avatar frame.
		"""
		return self._data.get('frameType')
		
	@property
	def frame_id(self) -> str:
		"""
		`frame_id` - The frame id of the avatar frame.
		
		`Returns`:
		- `str` - The frame id of the avatar frame.
		"""
		return self._data.get('frameId')

class InfluencerInfoNotFound:
	"""`InfluencerInfoNotFound` - Class representing a influencer info not found."""
	def __init__(self):
		pass

class InfluencerInfo:
	"""
	`InfluencerInfo` - Class representing a influencer info.

	`Attributes`:
	- `_data` - The data containing the influencer info.

	`Properties`:
	- `pinned` - The pinned status of the influencer info.
	- `created_time` - The created time of the influencer info.
	- `fans_count` - The fans count of the influencer info.
	- `monthly_fee` - The monthly fee of the influencer info.
	"""
	def __init__(self, data: dict):
		self._data = data

	@property
	def pinned(self) -> bool:
		"""
		`pinned` - The pinned status of the influencer.

		`Returns`:
		- `bool` - The pinned status of the influencer info.
		"""
		return self._data.get('pinned')

	@property
	def created_time(self) -> int:
		"""
		`created_time` - The influencer created time.

		`Returns`:
		- `int` - The created time of the influencer.
		"""
		return self._data.get('createdTime')

	@property
	def fans_count(self) -> int:
		"""
		`fans_count` - The fans count of the of the influencer.

		`Returns`:
		- `int` - The fans count of the influencer.
		"""
		return self._data.get('fansCount')

	@property
	def monthly_fee(self) -> int:
		"""
		`monthly_fee` - The monthly fee of the influencer.

		`Returns`:
		- `int` - The monthly fee of the influencer.
		"""
		return self._data.get('monthlyFee')

class CustomTitle:
    def __init__(self, data: dict):
        self._data = data

    @property
    def color(self) -> str:
        return self._data.get('color')

    @property
    def title(self) -> str:
        return self._data.get('title')

class UserExtensionsNotFound:
	"""`UserExtensionsNotFound` - Class representing a user extensions not found."""
	def __init__(self):
		pass		

class UserExtensions:
	def __init__(self, data: dict):
		self._data = data

	@property
	def privilege_of_comment_on_user_profile(self) -> int:
		return self._data.get('privilegeOfCommentOnUserProfile')
		
	@property
	def style(self) -> str:
		return self._data.get('style')
		
	@property
	def title_names(self) -> List[str]:
		titles_found = self._data.get('customTitles')
		return [title['title'] for title in titles_found] if titles_found else []

	@property
	def title_colors(self) -> List[str]:
		titles_found = self._data.get('customTitles')
		return [title['color'] for title in titles_found] if titles_found else []

	@property
	def titles(self) -> List[dict]:
		return self._data.get('customTitles', [])

	@property
	def privilege_of_chat_invite_request(self) -> int:
		return self._data.get('privilegeOfChatInviteRequest')

class MoodStickerNotFound:
	"""`MoodStickerNotFound` - Class representing a mood sticker not found."""
	def __init__(self):
		pass

class MoodSticker:
	"""
	`mood_sticker` - Mood sticker the user has set.

	`Returns:` MoodSticker | MoodStickerNotFound

	`Example`:
	```py
	>>> user = bot.community.fetch_user(0000-0000-0000-0000)
	>>> if isinstance(user.mood_sticker, MoodSticker):
	...	 print(user.mood_sticker.name)
	... else:
	...	 print('This user has no mood sticker set.')
	```
	"""
	def __init__(self, data: dict):
		self._data = data

	@property
	def status(self) -> int:
		"""
		`status` - is the status of request.

		`Returns`:
		- `int` - is the status of request.
		"""
		return self._data.get('status')

	@property
	def icon_v2(self) -> str:
		"""
		`icon_v2` - The icon v2 of the mood sticker.

		`Returns`:
		- `str` - The icon v2 of the mood sticker.
		"""
		return self._data.get('iconV2')

	@property
	def name(self) -> str:
		"""
		`name` - The name of the mood sticker.

		`Returns`:
		- `str` - The name of the mood sticker.
		"""
		return self._data.get('name')

	@property
	def sticker_id(self) -> int:
		"""
		`sticker_id` - The sticker id of the mood sticker.

		`Returns`:
		- `int` - The sticker id of the mood sticker.
		"""
		return self._data.get('stickerId')

	@property
	def small_icon_v2(self) -> str:
		"""
		`small_icon_v2` - The small icon v2 of the mood sticker.

		`Returns`:
		- `str` - The small icon v2 of the mood sticker.
		"""
		return self._data.get('smallIconV2')

	@property
	def small_icon(self) -> str:
		"""
		`small_icon` - The small icon of the mood sticker.

		`Returns`:
		- `str` - The small icon of the mood sticker.
		"""
		return self._data.get('smallIcon')

	@property
	def sticker_collection_id(self) -> int:
		"""
		`sticker_collection_id` - The sticker collection id of the mood sticker.

		`Returns`:
		- `int` - The sticker collection id of the mood sticker.
		"""
		return self._data.get('stickerCollectionId')

	@property
	def medium_icon(self) -> str:
		"""
		`medium_icon` - The medium icon of the mood sticker.

		`Returns`:
		- `str` - The medium icon of the mood sticker.
		"""
		return self._data.get('mediumIcon')

	@property
	def extensions(self) -> str:
		"""
		`extensions` - The extensions of the mood sticker.

		`Returns`:
		- `str` - The extensions of the mood sticker.
		"""
		return self._data.get('extensions')

	@property
	def used_count(self) -> int:
		"""
		`used_count` - The used count of the mood sticker.

		`Returns`:
		- `int` - The used count of the mood sticker.
		"""
		return self._data.get('usedCount')

	@property
	def medium_icon_v2(self) -> str:
		"""
		`medium_icon_v2` - The medium icon v2 of the mood sticker.

		`Returns`:
		- `str` - The medium icon v2 of the mood sticker.
		"""
		return self._data.get('mediumIconV2')

	@property
	def created_time(self) -> int:
		"""
		`created_time` - The created time of the mood sticker.

		`Returns`:
		- `int` - The created time of the mood sticker.
		"""
		return self._data.get('createdTime')

	@property
	def icon(self) -> str:
		"""
		`icon` - The icon of the mood sticker.

		`Returns`:
		- `str` - The icon of the mood sticker.
		"""
		return self._data.get('icon')

class UserProfile:
	"""
	`UserProfile` - Class representing a user profile.

	`Attributes`:
	- `_data` - The data containing the user profile.
	
	`Properties`:
	- `status` - is the status of request.
	- `mood_sticker` - Mood sticker the user has set.
	- `wiki_count` - The amount of wiki the user has created.
	- `consecutive_check_in_days` - The amount of consecutive days the user has checked in.
	- `uid` - The user id of the user.
	- `modified_time` - The time the user profile was last modified.
	- `following_status` - The following status of the user.
	- `online_status` - The online status of the user.
	- `account_membership_status` - The account membership status of the user.
	- `is_global` - Is the user a global user.
	- `avatar_frame_id` - The avatar frame id of the user.
	- `reputation` - The reputation of the user.
	- `posts_count` - The amount of posts the user has created.
	- `avatar_frame` - The avatar frame of the user.
	- `members_count` - The amount of members the user has.
	- `nickname` - The nickname of the user.
	- `media_list` - The media list of the user.
	- `icon` - The icon of the user.
	- `is_nickname_verified` - Is the nickname of the user verified.
	- `mood` - The mood of the user.
	- `level` - The level of the user.
	- `notification_subscription_status` - The notification subscription status of the user.
	- `settings` - The settings of the user.
	- `push_enabled` - Is the push enabled of the user.
	- `membership_status` - The membership status of the user.
	- `influencer_info` - The influencer info of the user.
	- `content` - The user's profile content.
	- `follower_count` - The amount of followers the user has.
	- `role` - The role of the user.
	- `comments_count` - The amount of comments the user has on their wall.
	- `ndc_id` - The community id the user is in.
	- `created_time` - The time the user was created.
	- `extensions` - The extensions of the user.
	- `stories_count` - The amount of stories the user has created.
	- `blogs_count` - The amount of blogs the user has created.
	- `is_user_hidden` - Is the user hidden.
	"""
	def __init__(self, data: dict):
		self._data = data.get('userProfile', data)
	
	@property
	def status(self) -> int:
		"""
		`status` - is the status of request.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.status)
		```
		"""
		return self._data.get('status')

	@property
	def mood_sticker(self) -> Union[MoodSticker, MoodStickerNotFound]:
		"""
		`mood_sticker` - Mood sticker the user has set.

		`Returns:` MoodSticker | MoodStickerNotFound

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> if isinstance(user.mood_sticker, MoodSticker):
		...	 print(user.mood_sticker.name)
		... else:
		...	 print('This user has no mood sticker set.')
		```
		"""
		moodSticker = self._data.get('moodSticker')
		return MoodSticker(moodSticker) if moodSticker else MoodStickerNotFound()
		
	@property
	def wiki_count(self) -> int:
		"""
		`wiki_count` - The amount of wiki the user has created.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.wiki_count)
		```
		"""
		return self._data.get('itemsCount')

	@property
	def consecutive_check_in_days(self) -> int:
		"""
		`consecutiveCheckInDays` - The amount of consecutive days the user has checked in.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.consecutive_check_in_days)
		```
		"""
		return self._data.get('consecutiveCheckInDays')

	@property
	def uid(self) -> str:
		"""
		`uid` - The user's uid.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.uid)
		```
		"""
		return self._data.get('uid')

	@property
	def userId(self) -> str:
		"""
		`userId` - The user's uid.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.userId)
		```
		"""
		return self.uid

	@property
	def aminoId(self) -> str:
		"""
		`aminoId` - The user's amino id.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.aminoId)
		```
		"""
		return self._data.get('aminoId')

	@property
	def modified_time(self) -> str:
		"""
		`modifiedTime` - The time the user's profile was last modified.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.modified_time)
		```
		"""
		return self._data.get('modifiedTime')

	@property
	def following_status(self) -> int:
		"""
		`followingStatus` - Whether the user is following the current user.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.following_status)
		```
		"""
		return self._data.get('followingStatus')

	@property
	def online_status(self) -> int:
		"""
		`onlineStatus` - The user's online status.
		
		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.online_status)
		```
		"""
		return self._data.get('onlineStatus')

	@property
	def account_membership_status(self) -> int:
		"""
		`account_membership_status` - The user's account membership status.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.account_membership_status)
		```
		"""
		return self._data.get('accountMembershipStatus')

	@property
	def is_global(self) -> bool:
		"""
		`is_global` - Whether the user is a global user.

		`Returns:` bool | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.is_global)
		```
		"""
		return self._data.get('isGlobal')

	@property
	def avatar_frame_id(self) -> str:
		"""
		`avatar_frame_id` - The user's avatar frame id.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.avatar_frame_id)
		```
		"""
		return self._data.get('avatarFrameId')

	@property
	def fan_club_list(self) -> list:
		"""
		`fan_club_list` - The user's fan club list.

		`Returns:` list | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.fan_club_list)
		```
		"""
		return self._data.get('fanClubList')

	@property
	def reputation(self) -> int:
		"""
		`reputation` - The user's reputation.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.reputation)
		```
		"""
		return self._data.get('reputation')

	@property
	def posts_count(self) -> int:
		"""
		`posts_count` - The amount of posts the user has created.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.posts_count)
		```
		"""
		return self._data.get('postsCount')

	@property
	def avatar_frame(self) -> Union[AvatarFrame, AvatarFrameNotFound]:
		"""
		`avatar_frame` - The user's avatar frame.
		
		`Returns:` dict | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> if isinstance(user.avatar_frame, AvatarFrame):
		...     print(user.avatar_frame.name)
		... else:
		...     print('Avatar frame not found.')
		```
		"""
		return AvatarFrame(self._data.get('avatarFrame'))

	@property
	def follower_count(self) -> int:
		"""
		`follower_count` - The amount of followers the user has.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.follower_count)
		```
		"""
		return self._data.get('membersCount')	

	@property
	def nickname(self) -> str:
		"""
		`nickname` - The user's nickname.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.nickname)
		```
		"""
		return self._data.get('nickname')

	@property
	def username(self) -> str:
		"""
		`username` - The user's username.
		
		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.username)
		```
		"""
		return self.nickname

	@property
	def media_list(self) -> list:
		"""
		`mediaList` - The user's media list.

		`Returns:` list | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.media_list)
		```
		"""
		return self._data.get('mediaList')

	@property
	def icon(self) -> str:
		"""
		`icon` - The user's icon.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.icon)
		```
		"""
		return self._data.get('icon')

	@property
	def avatar(self) -> str:
		"""
		`avatar` - The user's avatar.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.avatar)
		```
		"""
		return self.icon

	@property
	def is_nickname_verified(self) -> bool:
		"""
		`isNicknameVerified` - The user's nickname verification status.

		`Returns:` bool | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.is_nickname_verified)
		```
		"""
		return self._data.get('isNicknameVerified')

	@property
	def mood(self) -> str:
		"""
		`mood` - The user's mood.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.mood)
		```
		"""
		return self._data.get('mood')

	@property
	def level(self) -> int:
		"""
		`level` - The user's level.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.level)
		```
		"""
		return self._data.get('level')

	@property
	def push_enabled(self) -> bool:
		"""
		`pushEnabled` - The user's push notification status.

		`Returns:` bool | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.push_enabled)
		```
		"""
		return self._data.get('pushEnabled')

	@property
	def membership_status(self) -> int:
		"""
		`membershipStatus` - The user's membership status.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.membership_status)
		```
		"""
		return self._data.get('membershipStatus')

	@property
	def influencer_info(self) -> Union[InfluencerInfo, InfluencerInfoNotFound]:
		"""
		`influencerInfo` - The user's influencer info.

		`Returns:` dict | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> if isinstance(user.influencer_info, InfluencerInfo):
		...     print(user.influencer_info)
		... else:
		...     print("User influencer info not found.")
		```
		"""
		return InfluencerInfo(self._data.get('influencerInfo'))

	@property
	def content(self) -> str:
		"""
		`content` - The user's profile content.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.content)
		```
		"""
		return self._data.get('content')

	@property
	def following_count(self) -> int:
		"""
		`followingCount` - The amount of users the user is following.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.following_count)
		```
		"""
		return self._data.get('joinedCount')

	@property
	def role(self) -> int:
		"""
		`role` - The user's role.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.role)
		```
		"""
		return self._data.get('role')

	@property
	def comments_count(self) -> int:
		"""
		`commentsCount` - The amount of comments the user has on their wall.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.comments_count)
		```
		"""
		return self._data.get('commentsCount')

	@property
	def ndcId(self) -> int:
		"""
		`ndcId` - The community the user is in.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.ndcId)
		```
		"""
		return self._data.get('ndcId')

	@property
	def comId(self) -> int:
		"""
		`comId` - The community the user is in.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.comId)
		```
		"""
		return self._data.get('ndcId')

	@property
	def created_time(self) -> str:
		"""
		`createdTime` - The time the user was created.

		`Returns:` str | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.created_time)
		```
		"""
		return self._data.get('createdTime')
	
	@property
	def is_user_hidden(self) -> bool:
		"""
		`isUserHidden` - Whether the user is hidden.

		`Returns:` bool | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)	
		>>> print(f"Is user hidden: {user.is_user_hidden}")
		```
		"""
		if extensions := self._data.get('extensions'):
			return extensions.get('hideUserProfile', False)
		return False
	
	@property
	def is_user_banned(self) -> bool:
		"""
		`isUserBanned` - Whether the user is banned.

		`Returns:` bool | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)	
		>>> print(f"Is user banned: {user.is_user_banned}")
		```
		"""
		if extensions := self._data.get('extensions'):
			return bool(extensions.get('__disabledTime__', False))
		return False

	@property
	def extensions(self) -> Union[UserExtensions, UserExtensionsNotFound]:
		"""
		`extensions` - The user's extensions.

		`Returns:` dict | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> if isinstance(user.extensions, UserExtensions):
		...     print(user.extensions)
		... else:
		...     print("No extensions found.")
		```
		"""
		return UserExtensions(self._data.get('extensions'))

	@property
	def visit_privacy(self) -> int:
		"""
		`visitPrivacy` - The user's visit privacy.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.visit_privacy)
		```
		"""
		return self._data.get('visitPrivacy')

	@property
	def stories_count(self) -> int:
		"""
		`storiesCount` - The amount of stories the user has.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.stories_count)
		```
		"""
		return self._data.get('storiesCount')

	@property
	def blogs_count(self) -> int:
		"""
		`blogsCount` - The amount of blogs the user has.

		`Returns:` int | None

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.blogs_count)
		```
		"""
		return self._data.get('blogsCount')

	def json(self) -> dict:
		"""
		`json` - The json response from the api.
		
		`Returns:` dict

		`Example`:
		```py
		>>> user = bot.community.fetch_user(0000-0000-0000-0000)
		>>> print(user.json())
		```
		"""
		return self._data

class OnlineMembers:
	"""
	`OnlineMembers` - The online members of a community.
	
	`Attributes`:
	- `_data` - The json response from the api.
	- `_user` - The user's data.

	`Properties`:
	- `topic` - The community's topic.
	- `ndcId` - The community's id.
	- `comId` - The community's id.
	- `users_online` - The amount of users online.
	- `is_guest` - If the user is a guest.
	- `uid` - The user's id.
	- `userId` - The user's id.
	- `status` - The user's status.
	- `icon` - The user's icon.
	- `avatar` - The user's icon.
	- `reputation` - The user's reputation.
	- `role` - The user's role.
	- `nickname` - The user's nickname.
	- `username` - The user's nickname.
	- `level` - The user's level.
	- `extensions` - The user's extensions.
	- `account_membership_status` - The user's account membership status.
	- `avatar_frame_id` - The user's avatar frame id.
	- `avatar_frame` - The user's avatar frame.
	- `is_nickname_verified` - If the user's nickname is verified.
	- `json` - The json response from the api.
	"""
	def __init__(self, data: dict):
		self._data = data.get('o', data)
		self._user: dict = self._data.get('userProfileList')[0]

	@property
	def topic(self) -> str:
		"""
		`topic` - The community's topic.
		
		`Returns:` str | None
		"""
		return self._data.get('topic')
	
	@property
	def ndcId(self) -> str:
		"""
		`ndcId` - The community's id.

		`Returns:` str | None
		"""
		return self._data.get('ndcId')

	@property
	def comId(self) -> str:
		"""
		`comId` - The community's id.
		
		`Returns:` str | None
		"""
		return self.ndcId

	@property
	def users_online(self) -> int:
		"""
		`usersOnline` - The amount of users online.
		
		`Returns:` int | None
		"""
		return self._data.get('userProfileCount')

	@property
	def is_guest(self) -> bool:
		"""
		`isGuest` - If the user is a guest.
		
		`Returns:` bool | None
		"""
		return self._user.get('isGuest')

	@property
	def uid(self) -> str:
		"""
		`uid` - The user's id.
		
		`Returns:` str | None
		"""
		return self._user.get('uid')

	@property
	def userId(self) -> str:
		"""
		`userId` - The user's id.
		
		`Returns:` str | None
		"""
		return self.uid

	@property
	def status(self) -> str:
		"""
		`status` - The user's status.
		
		`Returns:` str | None
		"""
		return self._user.get('status')

	@property
	def icon(self) -> str:
		"""
		`icon` - The user's icon.
		
		`Returns:` str | None
		"""
		return self._user.get('icon')

	@property
	def avatar(self) -> str:
		"""
		`avatar` - The user's icon.
		
		`Returns:` str | None
		"""
		return self.icon

	@property
	def reputation(self) -> int:
		"""
		`reputation` - The user's reputation.
		
		`Returns:` int | None
		"""
		return self._user.get('reputation')

	@property
	def role(self) -> str:
		"""
		`role` - The user's role.
		
		`Returns:` str | None
		"""
		return self._user.get('role')

	@property
	def nickname(self) -> str:
		"""
		`nickname` - The user's nickname.
		
		`Returns:` str | None
		"""
		return self._user.get('nickname')

	@property
	def username(self) -> str:
		"""
		`username` - The user's nickname.
		
		`Returns:` str | None
		"""
		return self.nickname

	@property
	def level(self) -> int:
		"""
		`level` - The user's level.
		
		`Returns:` int | None
		"""
		return self._user.get('level')

	@property
	def extensions(self) -> UserExtensions:
		"""
		`extensions` - The user's extensions.
		
		`Returns:` UserExtensions | None
		
		`Example:`
		```py
		>>> user = OnlineUser(...)
		>>> if isinstance(user.extensions, UserExtensions):
		...     print(user.extensions.(...))
		... else:
		...     print('No extensions')
		```
		"""
		return UserExtensions(self._user.get('extensions'))

	@property
	def account_membership_status(self) -> str:
		"""
		`accountMembershipStatus` - The user's account membership status.

		`Returns:` str | None
		"""
		return self._user.get('accountMembershipStatus')

	@property
	def avatar_frame_id(self) -> str:
		"""
		`avatarFrameId` - The user's avatar frame id.

		`Returns:` str | None
		"""
		return self._user.get('avatarFrameId')

	@property
	def avatar_frame(self) -> AvatarFrame:
		"""
		`avatarFrame` - The user's avatar frame.

		`Returns:` AvatarFrame | None

		`Example:`
		```py
		>>> user = OnlineUser(...)
		>>> if isinstance(user.avatar_frame, AvatarFrame):
		...     print(user.avatar_frame.(...))
		... else:
		...     print('No avatar frame')
		```
		"""
		return AvatarFrame(self._user.get('avatarFrame'))

	@property
	def is_nickname_verified(self) -> bool:
		"""
		`isNicknameVerified` - If the user's nickname is verified.

		`Returns:` bool | None
		"""
		return self._user.get('isNicknameVerified')

	def json(self):
		"""
		`Returns:`
		- Api response in json format.
		"""
		return self._data

class UserProfileList:
	def __init__(self, data: Union[dict, list]) -> None:

		if isinstance(data, dict):
			self._data: List[dict] = data.get("userProfileList", data)
		else:
			self._data: List[dict] = data

		parser:							List[UserProfile] = [UserProfile(x) for x in self._data]
		self.status:					List[int] = [x.status for x in parser]
		self.mood_sticker:				List[MoodSticker] = [x.mood_sticker for x in parser]
		self.wiki_count:				List[int] = [x.wiki_count for x in parser]
		self.consecutive_check_in_days:	List[int] = [x.consecutive_check_in_days for x in parser]
		self.uid:						List[str] = [x.uid for x in parser]
		self.userId:					List[str] = self.uid
		self.modified_time:				List[str] = [x.modified_time for x in parser]
		self.following_status:			List[int] = [x.following_status for x in parser]
		self.online_status:				List[int] = [x.online_status for x in parser]
		self.account_membership_status:	List[int] = [x.account_membership_status for x in parser]
		self.is_global:					List[bool] = [x.is_global for x in parser]
		self.avatar_frame_id:			List[str] = [x.avatar_frame_id for x in parser]
		self.fan_club_list:				List[list] = [x.fan_club_list for x in parser]
		self.reputation:				List[int] = [x.reputation for x in parser]
		self.posts_count:				List[int] = [x.posts_count for x in parser]
		#self.avatar_frame:				List[AvatarFrameList] = [AvatarFrameList(x.avatar_frame.json()) for x in parser]	
		self.follower_count:			List[int] = [x.follower_count for x in parser]
		self.nickname:					List[str] = [x.nickname for x in parser]
		self.username:					List[str] = self.nickname
		self.media_list:				List[list] = [x.media_list for x in parser]
		self.icon:						List[str] = [x.icon for x in parser]
		self.avatar:					List[str] = self.icon
		self.is_nickname_verified:		List[bool] = [x.is_nickname_verified for x in parser]
		self.mood:						List[str] = [x.mood for x in parser]
		self.level:						List[int] = [x.level for x in parser]
		self.pushEnabled:				List[bool] = [x.push_enabled for x in parser]
		self.membership_status:			List[int] = [x.membership_status for x in parser]
		#self.influencer_info:			list = [x.influencer_info for x in parser]
		self.content:					List[str] = [x.content for x in parser]
		self.following_count:			List[int] = [x.following_count for x in parser]
		self.role:						List[int] = [x.role for x in parser]
		self.comments_count:			List[int] = [x.comments_count for x in parser]
		self.ndcId:						List[int] = [x.ndcId for x in parser]
		self.comId:						List[int] = self.ndcId
		self.created_time:				List[str] = [x.created_time for x in parser]
		#self.extensions:				List[UserExtensionsList] = [UserExtensionsList(self._data.get("extensions") for y in self._data) for x in parser]
		self.visit_privacy:				List[int] = [x.visit_privacy for x in parser]
		self.stories_count:				List[int] = [x.stories_count for x in parser]
		self.blogs_count:				List[int] = [x.blogs_count for x in parser]
		self.is_user_hidden:			List[bool] = [x.is_user_hidden for x in parser]
		self.is_user_banned:			List[bool] = [x.is_user_banned for x in parser]

class Pagging:
    def __init__(self, data: dict):
        self._data = data.get("paging") if isinstance(data, dict) else data
        self.prev_page_token = None
        self.next_page_token = None

        if isinstance(data, dict):
            self.prev_page_token: Union[str, None] = self._data.get("prevPageToken")
            self.next_page_token: Union[str, None] = self._data.get("nextPageToken")
    
    def json(self) -> dict:
        return self._data

class FollowerList:
    def __init__(self, data: dict):
        self._data = data
        self.paging = None
        self.members = None
        
        if isinstance(data, dict):
            self.paging = Pagging(self._data)
            self.members = UserProfileList(self._data)
    
    def json(self) -> dict:
        return self._data


from typing import List, Union


class StickerAuthor:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data
        except Exception:
            self.data = None

    def _check_author(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper
    
    @property
    @_check_author
    def is_global(self) -> bool:
        """Returns True if the author is global, False if not."""
        return self.data.get("isGlobal")
    
    @property
    @_check_author
    def comId(self) -> int:
        """Returns the author's community ID."""
        return self.data.get("ndcId")
    
    @property
    @_check_author
    def role(self) -> int:
        """Returns the author's role."""
        return self.data.get("role")
    
    @property
    @_check_author
    def is_nickname_verified(self) -> bool:
        """Returns True if the author's nickname is verified, False if not."""
        return self.data.get("isNicknameVerified")
    
    @property
    @_check_author
    def following_status(self) -> int:
        """Returns the author's following status."""
        return self.data.get("followingStatus")
    
    @property
    @_check_author
    def followers(self) -> int:
        """Returns the author's follower count."""
        return self.data.get("membersCount")
    
    @property
    @_check_author
    def membership_status(self) -> int:
        """Returns the author's membership status."""
        return self.data.get("membershipStatus")
    
    @property
    @_check_author
    def status(self) -> int:
        """Returns the author's status."""
        return self.data.get("status")
    
    @property
    @_check_author
    def account_membership_status(self) -> int:
        """Returns the author's account membership status."""
        return self.data.get("accountMembershipStatus")
    
    @property
    @_check_author
    def reputation(self) -> int:
        """Returns the author's reputation."""
        return self.data.get("reputation")
    
    @property
    @_check_author
    def level(self) -> int:
        """Returns the author's level."""
        return self.data.get("level")
    
    @property
    @_check_author
    def icon(self) -> str:
        """Returns the author's icon."""
        return self.data.get("icon")
    
    @property
    @_check_author
    def uid(self) -> str:
        """Returns the author's UID."""
        return self.data.get("uid")
    
    @property
    @_check_author
    def nickname(self) -> str:
        """Returns the author's nickname."""
        return self.data.get("nickname")
    
    def json(self) -> dict:
        """Returns the author's data as a dictionary."""
        return self.data


class OriginalCommunity:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data
        except Exception:
            self.data = None

    def _check_community(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper
    
    @property
    @_check_community
    def status(self) -> int:
        """Returns the community's status."""
        return self.data.get("status")
    
    @property
    @_check_community
    def icon(self) -> str:
        """Returns the community's icon."""
        return self.data.get("icon")
    
    @property
    @_check_community
    def endpoint(self) -> str:
        """Returns the community's endpoint."""
        return self.data.get("endpoint")
    
    @property
    @_check_community
    def name(self) -> str:
        """Returns the community's name."""
        return self.data.get("name")
    
    @property
    @_check_community
    def comId(self) -> int:
        """Returns the community's ndc id."""
        return self.data.get("ndcId")
    
    def json(self) -> dict:
        """Returns the community's json."""
        return self.data


class StickerExtensions:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data
        except Exception:
            self.data = None

    def _check_extension(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper
    
    @property
    @_check_extension
    def icon_source_sticker_id(self) -> str:
        """Returns the sticker's icon source sticker id."""
        return self.data.get("iconSourceStickerId")
    
    @property
    @_check_extension
    def original_author(self) -> StickerAuthor:
        """Returns the sticker's original author."""
        return StickerAuthor(self.data.get("originalAuthor"))
    
    @property
    @_check_extension
    def original_community(self) -> OriginalCommunity:
        """Returns the sticker's original community."""
        return OriginalCommunity(self.data.get("originalCommunity"))
    
    def json(self) -> dict:
        """Returns the sticker's extensions as a dictionary."""
        return self.data


class Sticker:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data
        except Exception:
            self.data = None

    def _check_sticker(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper

    @property
    @_check_sticker
    def ref_object_type(self) -> int:
        """Returns the sticker's reference object type."""
        return self.data.get("refObjectType")

    @property
    @_check_sticker
    def ref_object_id(self) -> str:
        """Returns the sticker's reference object ID."""
        return self.data.get("refObjectId")

    @property
    @_check_sticker
    def created_time(self) -> str:
        """Returns the sticker's creation time."""
        return self.data.get("createdTime")

    @property
    @_check_sticker
    def item_basic_info(self) -> dict:
        """Returns the sticker's basic info."""
        return self.data.get("itemBasicInfo")

    @property
    @_check_sticker
    def icon(self) -> str:
        """Returns the sticker's icon."""
        return self.item_basic_info.get("icon")

    @property
    @_check_sticker
    def name(self) -> str:
        """Returns the sticker's name."""
        return self.item_basic_info.get("name")

    @property
    @_check_sticker
    def item_restriction_info(self) -> dict:
        """Returns the sticker's restriction info."""
        return self.data.get("itemRestrictionInfo")

    @property
    @_check_sticker
    def discount_value(self) -> Union[int, None]:
        """Returns the sticker's discount value."""
        return self.item_restriction_info.get("discountValue")

    @property
    @_check_sticker
    def discount_status(self) -> int:
        """Returns the sticker's discount status."""
        return self.item_restriction_info.get("discountStatus")

    @property
    @_check_sticker
    def owner_uid(self) -> Union[str, None]:
        """Returns the sticker's owner UID."""
        return self.item_restriction_info.get("ownerUid")

    @property
    @_check_sticker
    def owner_type(self) -> int:
        """Returns the sticker's owner type."""
        return self.item_restriction_info.get("ownerType")

    @property
    @_check_sticker
    def restrict_type(self) -> int:
        """Returns the sticker's restriction type."""
        return self.item_restriction_info.get("restrictType")

    @property
    @_check_sticker
    def restrict_value(self) -> int:
        """Returns the sticker's restriction value."""
        return self.item_restriction_info.get("restrictValue")

    @property
    @_check_sticker
    def available_duration(self) -> Union[int, None]:
        """Returns the sticker's available duration."""
        return self.item_restriction_info.get("availableDuration")

    @property
    @_check_sticker
    def ref_object(self) -> dict:
        """Returns the sticker's reference object."""
        return self.data.get("refObject")

    @property
    @_check_sticker
    def modified_time(self) -> str:
        """Returns the sticker's modification time."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("modifiedTime")

    @property
    @_check_sticker
    def ownership_status(self) -> Union[int, None]:
        """Returns the sticker's ownership status."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("ownershipStatus")

    @property
    @_check_sticker
    def used_count(self) -> int:
        """Returns the sticker's used count."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("usedCount")

    @property
    @_check_sticker
    def extensions(self) -> Union[dict, None]:
        """Returns the sticker's extensions."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("extensions")

    @property
    @_check_sticker
    def available_community_ids(self) -> list:
        """Returns the sticker's available community IDs."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("availableNdcIds")

    @property
    @_check_sticker
    def status(self) -> int:
        """Returns the sticker's status."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("status")

    @property
    @_check_sticker
    def author(self) -> StickerAuthor:
        """Returns the sticker's author."""
        return StickerAuthor(self.ref_object.get("author"))

    @property
    @_check_sticker
    def is_new(self) -> bool:
        """Returns the sticker's new status."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("isNew")

    @property
    @_check_sticker
    def stickers_count(self) -> int:
        """Returns the sticker's count."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("stickersCount")

    @property
    @_check_sticker
    def restriction_info(self) -> dict:
        """Returns the sticker's restriction info."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("restrictionInfo")

    @property
    @_check_sticker
    def collection_id(self) -> str:
        """Returns the sticker's collection ID."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("collectionId")

    @property
    @_check_sticker
    def is_activated(self) -> bool:
        """Returns the sticker's activation status."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("isActivated")

    @property
    @_check_sticker
    def collection_type(self) -> int:
        """Returns the sticker's collection type."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("collectionType")

    @property
    @_check_sticker
    def uid(self) -> str:
        """Returns the sticker's UID."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("uid")

    @property
    @_check_sticker
    def small_icon(self) -> str:
        """Returns the sticker's small icon."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("smallIcon")

    @property
    @_check_sticker
    def description(self) -> str:
        """Returns the sticker's description."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("description")

    @property
    @_check_sticker
    def is_globally_available(self) -> bool:
        """Returns the sticker's global availability status."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("isGloballyAvailable")

    @property
    @_check_sticker
    def banner_url(self) -> str:
        """Returns the sticker's banner URL."""
        if self.ref_object is None:
            return None
        else:
            return self.ref_object.get("bannerUrl")
        
    def json(self) -> dict:
        """Returns the sticker's data in JSON format."""
        return self.data
    

class StickerList:
    def __init__(self, data: dict):
        try:
            self.data = data.get("storeItemList")
        except Exception:
            self.data = None

    def _check_stickers(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper
    
    def __iterator__(self):
        """
        Iterator function to iterate over the sticker list.
        """
        for sticker in self.data:
            yield Sticker(sticker)

    @property
    @_check_stickers
    def ref_object_type(self) -> List[int]:
        """Returns the sticker list's ref object type."""
        return [sticker.ref_object_type for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def ref_object_id(self) -> List[str]:
        """Returns the sticker list's ref object ID."""
        return [sticker.ref_object_id for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def created_time(self) -> List[str]:
        """Returns the sticker list's creation time."""
        return [sticker.created_time for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def item_basic_info(self) -> List[dict]:
        """Returns the sticker list's basic info."""
        return [sticker.item_basic_info for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def icon(self) -> List[str]:
        """Returns the sticker list's icon."""
        return [sticker.icon for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def name(self) -> List[str]:
        """Returns the sticker list's name."""
        return [sticker.name for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def item_restriction_info(self) -> List[dict]:
        """Returns the sticker list's restriction info."""
        return [sticker.item_restriction_info for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def discount_value(self) -> List[str]:
        """Returns the sticker list's discount value."""
        return [sticker.discount_value for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def discount_status(self) -> List[int]:
        """Returns the sticker list's discount status."""
        return [sticker.discount_status for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def owner_uid(self) -> List[str]:
        """Returns the sticker list's owner UID."""
        return [sticker.owner_uid for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def owner_type(self) -> List[int]:
        """Returns the sticker list's owner type."""
        return [sticker.owner_type for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def restrict_type(self) -> List[int]:
        """Returns the sticker list's restrict type."""
        return [sticker.restrict_type for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def restrict_value(self) -> List[int]:
        """Returns the sticker list's restrict value."""
        return [sticker.restrict_value for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def available_duration(self) -> List[str]:
        """Returns the sticker list's available duration."""
        return [sticker.available_duration for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def ownership_status(self) -> List[int]:
        """Returns the sticker list's ownership status."""
        return [sticker.ownership_status for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def is_owned(self) -> List[bool]:
        """Returns the sticker list's ownership status."""
        return [sticker.is_owned for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def is_new(self) -> List[bool]:
        """Returns the sticker list's new status."""
        return [sticker.is_new for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def is_activated(self) -> List[bool]:
        """Returns the sticker list's activation status."""
        return [sticker.is_activated for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def collection_id(self) -> List[str]:
        """Returns the sticker list's collection ID."""
        return [sticker.collection_id for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def collection_type(self) -> List[int]:
        """Returns the sticker list's collection type."""
        return [sticker.collection_type for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def uid(self) -> List[str]:
        """Returns the sticker list's UID."""
        return [sticker.uid for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def small_icon(self) -> List[str]:
        """Returns the sticker list's small icon."""
        return [sticker.small_icon for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def description(self) -> List[str]:
        """Returns the sticker list's description."""
        return [sticker.description for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def is_globally_available(self) -> List[bool]:
        """Returns the sticker list's global availability."""
        return [sticker.is_globally_available for sticker in self.__iterator__()]
    
    @property
    @_check_stickers
    def banner_url(self) -> List[str]:
        """Returns the sticker list's banner URL."""
        return [sticker.banner_url for sticker in self.__iterator__()]
    
    def json(self) -> dict:
        """Returns the sticker list's JSON data."""
        return self.data


class CommunitySticker:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data
        except Exception:
            self.data = None

    def _check_sticker(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper
    
    @property
    @_check_sticker
    def status(self) -> int:
        """Returns the sticker's status."""
        return self.data.get("status")
    
    @property
    @_check_sticker
    def is_activated(self) -> bool:
        """Returns True if the sticker is activated, False otherwise."""
        return self.data.get("isActivated")
    
    @property
    @_check_sticker
    def collection_type(self) -> int:
        """Returns the sticker's collection type."""
        return self.data.get("collectionType")
    
    @property
    @_check_sticker
    def is_new(self) -> bool:
        """Returns True if the sticker is new, False otherwise."""
        return self.data.get("isNew")
    
    @property
    @_check_sticker
    def banner_url(self) -> str:
        """Returns the sticker's banner url."""
        return self.data.get("bannerUrl")

    @property
    @_check_sticker
    def is_owned(self) -> bool:
        """Returns True if the sticker is owned, False otherwise."""
        return self.data.get("isOwned")
    
    @property
    @_check_sticker
    def used_count(self) -> int:
        """Returns the sticker's used count."""
        return self.data.get("usedCount")
    
    @property
    @_check_sticker
    def available_community_ids(self) -> list:
        """Returns a list of available community ids."""
        return self.data.get("availableNdcIds")
    
    @property
    @_check_sticker
    def icon(self) -> str:
        """Returns the sticker's icon url."""
        return self.data.get("icon")
    
    @property
    @_check_sticker
    def name(self) -> str:
        """Returns the sticker's name."""
        return self.data.get("name")

    @property
    @_check_sticker
    def collection_id(self) -> str:
        """Returns the sticker's collection id."""
        return self.data.get("collectionId")

    @property
    @_check_sticker
    def description(self) -> str:
        """Returns the sticker's description."""
        return self.data.get("description")

    @property
    @_check_sticker
    def author(self) -> StickerAuthor:
        """Returns the sticker's author."""
        return StickerAuthor(self.data.get("author"))

    @property
    @_check_sticker
    def ownership_info(self) -> dict:
        """Returns the sticker's ownership info."""
        return self.data.get("ownershipInfo")

    @property
    @_check_sticker
    def is_auto_renew(self) -> bool:
        """Returns True if the sticker is auto renew, False otherwise."""
        if self.owner_info is None:
            return None
        return self.owner_info.get("isAutoRenew")

    @property
    @_check_sticker
    def expired_time(self) -> str:
        """Returns the sticker's expired time."""
        if self.owner_info is None:
            return None
        return self.owner_info.get("expiredTime")

    @property
    @_check_sticker
    def extensions(self) -> StickerExtensions:
        """Returns the sticker's extensions."""
        return StickerExtensions(self.data.get("extensions"))

    @property
    @_check_sticker
    def created_time(self) -> str:
        """Returns the sticker's created time."""
        return self.data.get("createdTime")

    @property
    @_check_sticker
    def is_globally_available(self) -> bool:
        """Returns True if the sticker is globally available, False otherwise."""
        return self.data.get("isGloballyAvailable")

    
    @property   
    @_check_sticker
    def restriction_info(self) -> dict:
        """Returns the sticker's restriction info."""
        return self.data.get("restrictionInfo")

    @property
    @_check_sticker
    def discount_status(self) -> int:
        """Returns the sticker's discount status."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("discountStatus")

    @property
    @_check_sticker
    def owner_uid(self) -> str:
        """Returns the sticker's owner uid."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("ownerUid")

    @property
    @_check_sticker
    def owner_type(self) -> int:
        """Returns the sticker's owner type."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("ownerType")

    @property
    @_check_sticker
    def restrict_type(self) -> int:
        """Returns the sticker's restrict type."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("restrictType")

    @property
    @_check_sticker
    def restrict_value(self) -> int:
        """Returns the sticker's restrict value."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("restrictValue")

    @property
    @_check_sticker
    def available_duration(self) -> str:
        """Returns the sticker's available duration."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("availableDuration")

    @property
    @_check_sticker
    def discount_value(self) -> str:
        """Returns the sticker's discount value."""
        if self.restriction_info is None:
            return None
        return self.restriction_info.get("discountValue")

    def json(self) -> dict:
        """Returns the sticker's data as a dict."""
        return self.data
    

class CommunityStickerList:
    def __init__(self, data: dict) -> None:
        try:
            self.data = data.get("stickerCollectionList")
        except Exception:
            self.data = None

    def _check_sticker(F):
        def wrapper(*args, **kwargs):
            return None if args[0].data is None else F(*args, **kwargs)
        return wrapper

    def __iterator__(self):
        """
        Iterator function to iterate over the stickers in the sticker list.
        """
        for sticker in self.data:
            yield CommunitySticker(sticker)

    @property
    @_check_sticker
    def status(self) -> List[int]:
        """Returns the sticker's status."""
        return [sticker.status for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def is_activated(self) -> bool:
        """Returns True if the sticker is activated, False otherwise."""
        return [sticker.is_activated for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def collection_type(self) -> int:
        """Returns the sticker's collection type."""
        return [sticker.collection_type for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def is_new(self) -> bool:
        """Returns True if the sticker is new, False otherwise."""
        return [sticker.is_new for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def banner_url(self) -> str:
        """Returns the sticker's banner url."""
        return [sticker.banner_url for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def is_owned(self) -> bool:
        """Returns True if the sticker is owned, False otherwise."""
        return [sticker.is_owned for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def used_count(self) -> int:
        """Returns the sticker's used count."""
        return [sticker.used_count for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def available_community_ids(self) -> list:
        """Returns a list of available community ids."""
        return [sticker.available_community_ids for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def icon(self) -> str:
        """Returns the sticker's icon url."""
        return [sticker.icon for sticker in self.__iterator__()]
    
    @property
    @_check_sticker
    def name(self) -> str:
        """Returns the sticker's name."""
        return [sticker.name for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def collection_id(self) -> str:
        """Returns the sticker's collection id."""
        return [sticker.collection_id for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def description(self) -> str:
        """Returns the sticker's description."""
        return [sticker.description for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def ownership_info(self) -> dict:
        """Returns the sticker's ownership info."""
        return [sticker.ownership_info for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def is_auto_renew(self) -> bool:
        """Returns True if the sticker is auto renew, False otherwise."""
        if self.owner_info is None:
            return None
        return [sticker.is_auto_renew for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def expired_time(self) -> str:
        """Returns the sticker's expired time."""
        if self.owner_info is None:
            return None
        return [sticker.expired_time for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def extensions(self) -> StickerExtensions:
        """Returns the sticker's extensions."""
        return StickerExtensions(self.data.get("extensions"))

    @property
    @_check_sticker
    def created_time(self) -> str:
        """Returns the sticker's created time."""
        return [sticker.created_time for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def is_globally_available(self) -> bool:
        """Returns True if the sticker is globally available, False otherwise."""
        return [sticker.is_globally_available for sticker in self.__iterator__()]
    
    @property   
    @_check_sticker
    def restriction_info(self) -> dict:
        """Returns the sticker's restriction info."""
        return [sticker.restriction_info for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def discount_status(self) -> int:
        """Returns the sticker's discount status."""
        if self.restriction_info is None:
            return None
        return [sticker.discount_status for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def owner_uid(self) -> str:
        """Returns the sticker's owner uid."""
        if self.restriction_info is None:
            return None
        return [sticker.owner_uid for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def owner_type(self) -> int:
        """Returns the sticker's owner type."""
        if self.restriction_info is None:
            return None
        return [sticker.owner_type for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def restrict_type(self) -> int:
        """Returns the sticker's restrict type."""
        if self.restriction_info is None:
            return None
        return [sticker.restrict_type for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def restrict_value(self) -> int:
        """Returns the sticker's restrict value."""
        if self.restriction_info is None:
            return None
        return [sticker.restrict_value for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def available_duration(self) -> str:
        """Returns the sticker's available duration."""
        if self.restriction_info is None:
            return None
        return [sticker.available_duration for sticker in self.__iterator__()]

    @property
    @_check_sticker
    def discount_value(self) -> str:
        """Returns the sticker's discount value."""
        if self.restriction_info is None:
            return None
        return [sticker.discount_value for sticker in self.__iterator__()]

    def json(self) -> dict:
        """Returns the sticker's data as a dict."""
        return self.data


import sys


class ChatConsole:
    def __init__(self, console):
        """
        The ChatConsole class handles chat related operations in the application.

        :param console: An instance of the Console class for inter-component communication.
        :type console: Console
        """
        self.console = console
        self.stickers = {
            1: {"Laughing emoji": "e/f09f9882"},
            2: {"Nervous emoji": "e/f09f9885"},
            3: {"Nerd emoji": "e/f09fa493"},
            4: {"Sleep emoji": "e/f09f98b4"},
            5: {"Annoyed emoji": "e/f09f9892"},
            6: {"Sob emoji": "e/f09f98ad"},
            7: {"Angel emoji": "e/f09f9887"},
            8: {"Cowboy emoji": "e/f09fa4a0"},
            9: {"Clown emoji": "e/f09fa4a1"},
            10: {"Smirk emoji": "e/f09f988f"},

        }
        self.help_message = """
Chatroom Help:
    - Type any message to send it to the chat.
    - Type 'reply' to reply to a message.
    - Type 'sticker' to send a sticker.
    - Type 'leave' to leave the chat.
    - Type 'exit' to return to the main menu.
    - Type 'help' to see this message again.
    - Type 'clear' to clear the chat.
"""

    def join_public_chat(self):
        """
        Handles joining of a public chat.
        """
        if not self.console.bot.community_id:
            return self.console.on_error("You must select a community first.")

        self.console.print("Join Public Chat")
        self.console.print("""
    1. Join by Chat ID
    2. Join by Link
    3. Find Public Chats To Join
    4. Back
    """)
        choice = self.console.input(">>> ")
        self.console.print()

        try:
            if choice == "1":
                chat_id = self.console.input("Enter chat ID: ")
            elif choice == "2":
                chat_link = self.console.input("Enter chat link: ")
                chat_id = self.console.bot.community.fetch_object_id(chat_link)
            elif choice == "3":
                self.console.clear()
                self.console.print("Find Public Chats To Join")

                public_chats = self.console.bot.community.fetch_public_chats(size=25)
                for index, (chat_id, chat_title) in enumerate(zip(public_chats.chatId, public_chats.title)):
                    self.console.print(f"{index+1}. {chat_title}({chat_id})")

                self.console.print()
                choice = self.console.input(">>> ")
                self.console.print()

                try:
                    choice = int(choice)
                    if not (1 <= choice <= len(public_chats.chatId)):
                        raise ValueError
                    chat_id = public_chats.chatId[choice-1]
                except ValueError:
                    self.console.print("Invalid option. Please try again.")
                    return self.join_public_chat()

            elif choice == "4":
                return self.console.menu.display()

            else:
                self.console.print("Invalid option. Please try again.")
                return self.join_public_chat()

            self.console.bot.community.join_chat(chatId=chat_id)
            self.console.print("Joined chat successfully.")
            self.console.sleep(2)

        except Exception as e:
            self.console.print(f"Error: {e}")
            self.console.sleep(2)

        return self.console.menu.display()

    def my_chats(self):
        """
        Lists the user's chats and allows them to interact with selected chat.
        """
        if not self.console.bot.community_id:
            return self.console.on_error("You must select a community first.")
        self.print_chats()
        chat_id, chat_title = self.select_chat()
        if chat_id and chat_title:
            self.interact_with_chat(chat_id, chat_title)
        else:
            return self.console.menu.display()

    def print_chats(self):
        """
        Prints the user's chats.
        """
        self.console.print("My Chats")
        chats = self.console.bot.community.fetch_chats()
        for index, (chat_id, chat_title) in enumerate(zip(chats.chatId, chats.title)):
            if chat_title is None:
                chat_users = self.console.bot.community.fetch_chat_members(chat_id).members.nickname[:3]
                chat_title = ", ".join(chat_users) + "[Private Chat]"
            else:
                chat_title = f"{chat_title}[Public Chat]"
            self.console.print(f"{index+1}. {chat_title}({chat_id})")
        self.console.print("\nType 'back' to go back to the menu.\n")

    def select_chat(self):
        """
        Allows the user to select a chat.

        :return: Tuple containing chat ID and chat title, or (None, None) if back was selected.
        :rtype: Tuple[str, str]
        """
        choice = self.console.input(">>> ")
        self.console.print()
        if choice == "back":
            return None, None
        try:
            choice = int(choice)
            chats = self.console.bot.community.fetch_chats()
            if not (1 <= choice <= len(chats.chatId)):
                raise ValueError
            chat_id, chat_title = chats.chatId[choice-1], chats.title[choice-1]
            return chat_id, chat_title
        except ValueError:
            self.console.print("Invalid option. Please try again.")
            return self.select_chat()

    def interact_with_chat(self, chat_id: str, chat_title: str):
        """
        Allows the user to interact with the selected chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :param chat_title: Title of the chat.
        :type chat_title: str
        :return: None
        """
        self.replies = {}
        self.console.clear()
        self.console.print(f"Chat: {chat_title}")

        self.initiate_message_listener(chat_id)
        self.console.print(self.help_message)

        while self.handle_chat_interaction(chat_id, chat_title):
            self.console.sleep(2)

        self.console.print("\nReturning to Main Menu.\n")
        self.console.sleep(2)
        return self.console.menu.display()

    def initiate_message_listener(self, chat_id: str):
        """
        Initiates a message listener for the specified chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :return: None
        """
        counter = 0

        @self.console.bot._console_on_text_message()
        def on_message(ctx):
            nonlocal counter
            if ctx.chatId == chat_id:
                self.console.print()
                self.replies[counter] = ctx.message.messageId
                self.console.print(f"[{counter}] {ctx.author.nickname}: {ctx.message.content}")
                counter += 1
                sys.stdout.write(" "*self.console.indent_size + ">>> ")
                sys.stdout.flush()

    def handle_chat_interaction(self, chat_id: str, chat_title: str = None) -> bool:
        """
        Handles user input for interacting with the chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :return: True if the interaction should continue, False otherwise.
        :rtype: bool
        """
        try:
            message = self.console.input(">>> ")
            if message in {"exit", "quit", ""}:
                self.handle_leave(None)
                return False
            elif message == "reply":
                return self.handle_reply(chat_id)
            elif message == "sticker":
                return self.handle_sticker(chat_id)
            elif message == "leave":
                return self.handle_leave(chat_id)
            elif message == "help":
                self.console.print(self.help_message)
                return True
            elif message == "clear":
                self.console.clear()
                self.console.print(f"Chat: {chat_title}")
                return True
            else:
                self.console.bot.community.send_message(chatId=chat_id, content=message)
                self.console.print(f"You: {message}")
                return True
        except (EOFError):
            self.console.print("\n")
            sys.stdout.write(">>> ")
            sys.stdout.flush()
            return True

    def handle_reply(self, chat_id: str) -> bool:
        """
        Handles replying to a message in the chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :return: True if the interaction should continue, False otherwise.
        :rtype: bool
        """
        self.console.print("Enter the message ID of the message you want to reply to.")
        message_id = self.console.input(">>> ")
        if message_id in {"exit", "quit", ""}:
            return True
        try:
            message_id = int(message_id)
        except ValueError:
            self.console.print("Invalid message ID. Please try again.")
            return True
        if message_id not in self.replies:
            self.console.print("Invalid message ID. Please try again.")
            return True
        self.console.print("Enter your reply.")
        reply = self.console.input(">>> ")
        if reply in {"exit", "quit", ""}:
            return True
        self.console.bot.community.reply_message(chatId=chat_id, content=reply, messageId=self.replies[message_id])
        self.console.print(f"You: {reply}")
        return True

    def handle_sticker(self, chat_id: str) -> bool:
        """
        Handles sending a sticker in the chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :return: True if the interaction should continue, False otherwise.
        :rtype: bool
        """
        
        for key, value in self.stickers.items():
            self.console.print(f"[{key}] {list(value.keys())[0]}")

        self.console.print("Enter the key of the sticker you want to send.")
        self.console.print("Example: >>> 1")


        sticker_id = self.console.input(">>> ")
        if sticker_id in {"exit", "quit", ""}:
            return True
        
        try:
            self.console.bot.community.send_sticker(chatId=chat_id, stickerId=self.stickers[int(sticker_id)][list(self.stickers[int(sticker_id)].keys())[0]])
            self.console.print(f"You: {sticker_id}")
            return True
        except Exception:
            self.console.print("Invalid sticker ID. Please try again.")
            return True
            

    def handle_leave(self, chat_id: str) -> bool:
        """
        Handles leaving the chat.

        :param chat_id: ID of the chat.
        :type chat_id: str
        :return: False, as the user is leaving the chat.
        :rtype: bool
        """
        if chat_id:
            self.console.bot.community.leave_chat(chat_id)
            
        self.replies.clear()
        del self.console.bot._events["_console_text_message"]
        self.console.print("Left chat successfully.")
        self.console.sleep(2)
        return False



from .entities.chat_threads import *
from .entities.general import *
from .entities.messages import *
from .entities.wsevents import *
from .utilities.generate import *
from .entities.userprofile import *

from .console import *
from .socket import *
from .account import *
from .context import *
from .global_client import *
from .community import *
from .dispatcher import *
from .handle_queue import *
from .utilities.request_handler import *


from typing import Callable

class MessageDispatcher:
    """
    `MessageDispatcher` - Simple message dispatcher that allows you to register handlers for specific message types.
 
    `**Example**`

    ```py

    dispatcher = MessageDispatcher()

    message_type = 1000
    message = {"t": message_type, "d": {"foo": "bar"}}
    handler = lambda message: print(message)

    dispatcher.register(message_type, handler)
    dispatcher.handle(message)
    ```

    """
    def __init__(self):
        self.dispatch_table = {}

    def register(self, message_type: int, handler: Callable):
        self.dispatch_table[message_type] = handler

    def handle(self, message: dict):
        message_type = message.get("t")
        if message_type not in self.dispatch_table:
            return None
        self.dispatch_table[message_type](message)


class AsyncMessageDispatcher:
    """
    `AsyncMessageDispatcher` - Simple async message dispatcher that allows you to register handlers for specific message types.
 
    `**Example**`

    ```py

    dispatcher = AsyncMessageDispatcher()

    message_type = 1000
    message = {"t": message_type, "d": {"foo": "bar"}}
    handler = lambda message: print(message)

    dispatcher.register(message_type, handler)
    await dispatcher.handle(message)
    ```

    """
    def __init__(self):
        self.dispatch_table = {}

    def register(self, message_type: int, handler: Callable):
        self.dispatch_table[message_type] = handler

    async def handle(self, message: dict):
        message_type = message.get("t")
        if message_type not in self.dispatch_table:
            return
        await self.dispatch_table[message_type](message)


from pathlib import Path
from configparser import ConfigParser
from setuptools import setup, find_packages

config = ConfigParser()
config.read('setup.cfg')

setup(
    name="pymino",
    license="MIT",
    author="DarkyAmagai",
    author_email="darkyamagai@outlook.es",
    description="Easily create a bot for Amino Apps using a modern easy to use synchronous library.",
    long_description=(Path(__file__).parent / "README.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/forevercynical/pymino",
    version=config.get('metadata', 'version'),
    packages=find_packages(),
    install_requires=[
        "requests==2.32.3",
        "ujson==5.10.0",
        "colorama==0.4.6",
        "websocket-client==1.8.0",
        "diskcache==5.6.3",
        "aiohttp==3.12.2",
        "wsaccel==0.6.7"
    ],
    keywords=[
        "amino",
        "pymino",
        "narvii",
        "amino-api",
        "narvii-bots",
        "aminoapps",
        "amino-bot",
        "amino-bots"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8"
)
